@@ -16,7 +16,11 @@
 #include <afs/param.h>
 #endif
 
-RCSID("$Header: /cvs/openafs/src/rxkad/rxkad_common.c,v 1.8.2.1 2001/10/13 04:21:56 shadow Exp $");
+#ifdef AFS_SUN59_ENV
+#include <sys/time_impl.h>
+#endif
+
+RCSID("$Header: /cvs/openafs/src/rxkad/rxkad_common.c,v 1.8.2.4 2002/07/10 19:43:20 shadow Exp $");
 
 #ifdef KERNEL
 #ifndef UKERNEL
@@ -29,7 +33,7 @@ RCSID("$Header: /cvs/openafs/src/rxkad/rxkad_common.c,v 1.8.2.1 2001/10/13 04:21
 #include "../h/time.h"
 #ifndef AFS_LINUX22_ENV
 #include "../rpc/types.h"
-#include "../rpc/xdr.h"
+#include "../rx/xdr.h"
 #endif /* AFS_LINUX22_ENV */
 #else /* !UKERNEL */
 #include "../afs/sysincludes.h"
@@ -73,7 +77,7 @@ char *rxi_Alloc();
 #ifndef KERNEL
 #define osi_Time() time(0)
 #endif
-struct rxkad_stats rxkad_stats;
+struct rxkad_stats rxkad_stats = {0};
 
 /* this call sets up an endpoint structure, leaving it in *network* byte
  * order so that it can be used quickly for encryption.
