@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.7 2001/08/08 00:04:18 shadow Exp $");
+RCSID("$Header: /cvs/openafs/src/venus/fstrace.c,v 1.7.2.1 2002/04/21 21:05:46 zacheiss Exp $");
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -1594,6 +1594,11 @@ tryagain:
     {
 	if (!failed) {
 	    failed = 1;
+#if defined(AFS_OSF_ENV) && !defined(AFS_OSF20_ENV)
+	    catclose1 (catd);
+#else
+	    catclose (catd);
+#endif
 	    goto tryagain;
 	}
         sprintf ((char *) error_text, "status %08x (%s / %s)", 
