@@ -13,6 +13,11 @@
  *
  *------------------------------------------------------------------------*/
 
+#include <afsconfig.h>
+#include <afs/param.h>
+
+RCSID("$Header: /cvs/openafs/src/xstat/xstat_cm_test.c,v 1.5.2.1 2002/07/31 22:19:53 shadow Exp $");
+
 #include "xstat_cm.h"		/*Interface for xstat_cm module*/
 #include <cmd.h>		/*Command line interpreter*/
 #include <time.h>
@@ -446,6 +451,7 @@ print_cmCallStats()
     printf("\t%10d afs_GetCell\n",         cmp->callInfo.C_afs_GetCell);
     printf("\t%10d afs_GetCellByIndex\n",         cmp->callInfo.C_afs_GetCellByIndex);
     printf("\t%10d afs_GetCellByName\n",         cmp->callInfo.C_afs_GetCellByName);
+    printf("\t%10d afs_GetRealCellByIndex\n",         cmp->callInfo.C_afs_GetRealCellByIndex);
     printf("\t%10d afs_NewCell\n",         cmp->callInfo.C_afs_NewCell);
     printf("\t%10d CheckVLDB\n",         cmp->callInfo.C_CheckVLDB);
     printf("\t%10d afs_GetVolume\n",         cmp->callInfo.C_afs_GetVolume);
@@ -1277,7 +1283,7 @@ int RunTheTest(a_s)
 		    rn, curr_item->data);
 	    exit(-1);
 	}
-	bcopy(he->h_addr, &(CMSktArray[currCM].sin_addr.s_addr), 4);
+	memcpy(&(CMSktArray[currCM].sin_addr.s_addr), he->h_addr, 4);
 
 	/*
 	 * Move to the next CM name.
